/**
 * Project:  C_lesson2_ex1
 * File:     main.c
 * Author:   <your name>  <date created>
 *
 * This program demonstrates how to use the AVR-BED function library (lib_avrbed.a)
 * to initialize the LCD module, send it commands, and display text on it.
 *
 * The program also shows how to convert a number (integer variable) to a string
 * of printable digits for displaying on the LCD.
 * The toolchain (AVR GCC) provides additional library functions, e.g. itoa().
 */
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "lib_avrbed.h"
#include <util/delay.h>

// Text strings to display
char  AppTitle[] = "AVR-BED with LCD";
char  Blanks[] = "                ";     // array of 16 spaces


int  main(void)
{
    // Allocate an array of char's, i.e. a string variable...
    char  strBuffer[20];    // more than enough space for 16 chars
    int   iCount;           // local integer variable

    initialise_LCD();             // Initialize I/O ports and LCD module
    lcd_command(LCD_CLR);         // Send 1-byte command to clear LCD
    DDRC = DDRC & 0x07;           // Configure pins PC3..PC7 as inputs

    lcd_cursor_posn(0, 0);        // Set LCD cursor to upper LHS
    lcd_print_string(AppTitle);   // string to show on top line

    lcd_cursor_posn(1, 0);
    lcd_print_string("Your text here..");    // string to show on bottom line

    // Wait a few seconds so that we can read the text before proceeding.
    _delay_ms(3000);              // Delay function built into AVR GCC

    lcd_cursor_posn(1, 0);
    lcd_print_string(Blanks);     // Clear bottom line

    iCount = 10;  // initial value for countdown

    while (iCount != 0)
    {
        // convert number (iCount) to string of digits in strBuffer[]
        itoa(iCount, strBuffer, 10);

        // Display buffer contents on the bottom line (row = 1)
        lcd_cursor_posn(1, 6);
        lcd_print_string(strBuffer);

        // Wait a second before next value is displayed
        _delay_ms(1000);
        lcd_cursor_posn(1, 0);
        lcd_print_string(Blanks);  // Clear bottom line

        iCount = iCount - 1;  // decrement the counter (alt.  iCount--; )
    }

    lcd_cursor_posn(1, 6);
    lcd_print_string("BANG!");

}

