/*
 * File:     lesson2_ex3b_main.c
 * Author:   M.J.Bauer  2020
 *
 * This program demonstrates integer binary-to-decimal and hexadecimal conversion.
 *
 * A function uint2deca() is defined to convert an unsigned integer
 * variable (16 bits) into a string of 5 decimal digits stored in an array.
 *
 * Another function uint2hexa() is defined to convert an unsigned integer
 * variable (16 bits) into a string of 4 hexadecimal digits stored in an array.
 *
 * The main function calls uint2deca() to display test data in decimal;
 * it calls uint2hexa() to display test data in hex, prefixed with "0x".
 */
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "lib_avrXmini.h"  // or "lib_avrbed.h" for AVR^BED
#include <util/delay.h>

#define NUL  0    // define ASCII NUL code = 0

// Text strings to display
char  AppTitle[] = "Decimal  Hex";
char  Blanks[] = "                ";     // array of 16 spaces

// Function prototype declarations:
void  uint2deca(unsigned uint, char *result);
void  uint2hexa(unsigned uint, char *result);


int  main(void)
{
    unsigned  uval;       // local integer variable (unsigned!)
    char  digits[8];      // array of decimal ASCII digits (result string)

    initialise_LCD();             // Initialize I/O ports and LCD module
    lcd_command(LCD_CURSOR_OFF);  // Cursor OFF, display ON
    lcd_cursor_posn(0, 0);        // Set LCD cursor to upper LHS
    lcd_print_string(AppTitle);   // Show title on top line

    uval = 0xFFF;     // <<<<<<<<<<<<<<<<<<  Set integer value to display here
	
	// Call function to convert uval to a string of decimal digits:
	uint2deca(uval, digits);

	// Display the decimal string on the bottom line, beginning at column 0:
    lcd_cursor_posn(1, 0);
	lcd_print_string(digits);  // "digits" = (address of) output array
	
	// Call function to convert uval to a string of hex digits:
	uint2hexa(uval, digits);
	
	// Display the hex result string on the bottom line, beginning at column 9:
	lcd_cursor_posn(1, 9);
	lcd_print_string("0x");    // Hex number prefix
	lcd_print_string(digits);  // "digits" = (address of) output array

    while ( 1 )   // loop forever -- program "stops" here!
    {
        ;  // do nothing
    }
}


/*
* Function to convert an unsigned integer variable (16 bits) into a string of
* decimal digits (ASCII character codes), terminated with a 'NUL' code (0).
*
* Input argument:   uint = unsigned integer value (variable or constant)
* Output argument:  result = array where result string will be stored
*
* Note:     result[0] is the most significant digit (MSD), so that the array
*           elements will be in the correct order for display (MSD first).
*
* Example:  Let uint = 307.  The output array will be as follows...
*                   i  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | ...
*           result[i]  |'0'|'0'|'3'|'0'|'7'|NUL| X | ...   (X = not used)
*                       MSD             LSD
*/
void  uint2deca(unsigned uint, char *result)
{
	short  i = 4;   // index into array: result[]
	
	while (i >= 0)  // count backwards from i = 4 until i < 0
	{
		// Find the remainder of (uint / 10) and convert it to ASCII code:
		result[i] = (uint % 10) + '0';  
		
		uint = uint / 10;   // prepare next significant digit
		
		i = i - 1;   // Alternative:  i--;  (decrement i)
	}
	
	// Insert ASCII NUL code (0) to terminate the result string:
	result[5] = NUL;
}


/*
* Function to convert an unsigned integer variable (16 bits) into a string of
* hexadecimal digits (ASCII character codes), terminated with a 'NUL' code (0).
* ```````````
* Input argument:   uint = unsigned integer value (variable or constant)
* Output argument:  result = array where result string will be stored
*
* Note:     result[0] is the most significant digit (MSD), so that the array
*           elements will be in the correct order for display (MSD first).
*
* Example:  Let uint = 0x0F5A.  The output array will be as follows...
*                   i  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | ...
*           result[i]  |'0'|'F'|'5'|'A'|NUL| X | X | ...   (X = not used)
*                       MSD         LSD    
*/
void  uint2hexa(unsigned uint, char *result)
{
	short  i = 3;     // index into array: result[]
	BYTE   hexDigit;  // hex digit value (0..15), not ASCII char
	
	while (i >= 0)  // count backwards from i = 3 until i < 0
	{
		hexDigit = uint & 0x000F;    // Keep lower 4 bits, others = 0
		
		if (hexDigit <= 9)  result[i] = '0' + hexDigit;   // hexDigit <= 9
		else  result[i] = 'A' + (hexDigit - 10);          // hexDigit >= 10

		uint = uint >> 4;    // Shift next digit (4 bits) into lower 4 bits
		
		i = i - 1;   // Alternative:  i--;  (decrement i)
	}
	
	// Insert ASCII NUL code (0) to terminate the result string:
	result[4] = NUL;
}
