/**
 * File:  Lesson_3_Ex1_main.c
 *
 * This program uses the AVR on-chip ADC to measure the voltage on an input
 * pin (PC3) wired to the potentiometer on the AVR-BED.
 *
 * Author:   Mike Bauer, 2020
 * Version:  1.0
 */
#include <avr/io.h>
#define F_CPU  8000000UL     // CPU runs at 8 MHz

// These header files contain definitions needed by library functions:
#include <avr/interrupt.h>   // required for interrupt handlers
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include "lib_avrbed.h"  // def's for lib_avrbed.a


int  main(void)
{
	char  strNum[20];       // number converted to string
	int   adc_count;        // Raw ADC conversion count (0..1023)
	int   reading_mV;       // Voltage reading (0..5000 mV)
	int   last_reading = 9999;
	
	TC1_initialize();             // TC1 is set up for milliseconds() function
	GLOBAL_INT_ENABLE();

    lcd_initialise();             // Initialize I/O ports and LCD module
    lcd_command(LCD_CLR);         // Send 1-byte command to clear LCD
	lcd_command(LCD_CURSOR_OFF);  // Display ON, Cursor OFF
    DDRC = DDRC & 0x07;           // Make PC3..PC6 inputs
	
	lcd_cursor_posn(0, 0);
	lcd_print_string("ADC count: ");
	lcd_cursor_posn(1, 0);
	lcd_print_string("Voltage: ");

    while (TRUE)  // loop forever
    {
		adc_count = ADC_ReadInput(3);    // Read pot input ADC3/PC3

        if (adc_count != last_reading)   // reading has changed
		{
			lcd_cursor_posn(0, 11);
			lcd_print_string("     ");   // clear data field, top line
			lcd_cursor_posn(1, 9);
			lcd_print_string("       ");   // clear data, bottom line

			reading_mV = (5000 * (long) adc_count) / 1024;  // Convert to mV

			// Convert adc_count to string of decimal digits in array strNum
			itoa(adc_count, strNum, 10);
			lcd_cursor_posn(0, 11);
			lcd_print_string(strNum);

			// Convert reading_mV to string of decimal digits in array strNum
			itoa(reading_mV, strNum, 10);
			lcd_cursor_posn(1, 9);
			lcd_print_string(strNum);
			lcd_print_string(" mV");
		}

		last_reading = adc_count;
		_delay_ms(50);
    }
}

// end of file


