/*
 * Program to receive characters on UART0 RX line and display on LCD panel.
 *
 * Hardware platform:  AVR-BED or Atmel ATmega328P X-mini
 * Code library required:  "lib_avrbed.h" or "lib_avrXmini.h" (resp.)
 *
 * Test using PuTTY.  Set baud rate to 38400
 */
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "lib_avrbed.h"    // or "lib_avrXmini.h"

void  LCD_Scroll_Line(void);   // Function prototype

char  lineBuffer[20];   // array of received chars


int  main(void)
{
    BYTE  cpos = 0;   // LCD column index
    char  c;
    
    lcd_initialise(); 
    lcd_command(LCD_CURSOR_OFF);
    lcd_cursor_posn(0, 0);
    lcd_print_string("Activate PuTTY");
    lcd_cursor_posn(1, 0);

    UART_init(38400);
    UART_PutString("\n\n");   // Output 2 newlines, then startup message...
    UART_PutString("Activate PuTTY -- click anywhere in PuTTY window.\n");
    UART_PutString("Type on PC keyboard. Input should appear on AVR board LCD screen.\n");
    UART_PutString("Hit [Enter] to scroll to new line...\n");

    while ( TRUE )
    {
        if (UART_RxDataAvail())    // Check for received data available from UART
        {
            c = UART_GetByte();    // Fetch the received character
            
            if (c == '\r')         // if Enter key hit, i.e. if RX char is ASCII CR (0x0D)...
            {
                LCD_Scroll_Line();
                lcd_cursor_posn(1, 0);
                cpos = 0;
                UART_PutString("\r\n");   // Echo NEW-LINE to PuTTY
            }
            else if (c >= 0x20)    // else, if c is a printable character...
            {
                lcd_write_char(c);     // display char on LCD (bottom line)
                UART_PutByte(c);       // Echo input char back to PuTTY
                
                lineBuffer[cpos] = c;     // save char in buffer
                lineBuffer[cpos+1] = 0;   // append NUL terminator (ASCII NUL is 0)
                cpos++;
            }
            
            if (cpos >= 16)   // reached end of line?
            {
                LCD_Scroll_Line();
                lcd_cursor_posn(1, 0);
                cpos = 0;
                UART_PutString("\r\n");   // Echo NEW-LINE to PuTTY
            }
        }
    }
}


// Function to scroll LCD up 1 line and clear bottom line.
// Chars on bottom line (saved in buffer) are moved to top line.
//
void  LCD_Scroll_Line(void)
{
    // clear top line
    lcd_cursor_posn(0, 0);
    lcd_print_string("                ");
    
    // Copy chars from buffer to top line
    lcd_cursor_posn(0, 0); 
    lcd_print_string(lineBuffer);
    
    // clear bottom line
    lcd_cursor_posn(1, 0);
    lcd_print_string("                ");   
}

