/**
 * File:  AVR-poker-game-outline.c
 *
 * Author:   <your name>
 * Date:     <date originated>
 * Version:  1.0
 *
 * Pseudo-code outline for AVR X-mini poker game  (comment to be deleted)
 * Replace the dummy statements ( ; ) with actual code to make it work!
 * `````````````````````````````````````````````````````````````````````````
 * Reference:  'EEH115A Assignment Task A5 -- AVR Poker game'
 *
 * Peripherals (LCD, LEDs, push-button, etc) must be wired according to the
 * schematic diagram "AVR-X-mini-Peripheral-Wiring.pdf" which can be found
 * in the Swinburne AVR X-mini kit documentation pack (Zip file) on Canvas.
 *
 * Attn Students:  Add comments here to describe your program operation!
 * `````````````````````````````````````````````````````````````````````````
 *
 */
#include <avr/io.h>
#define F_CPU  16000000UL    // X-mini board CPU runs at 16 MHz

// These header files contain definitions needed by library functions:
#include <stdlib.h>
#include <string.h>
#include "lib_avrXmini.h"    // def's for X-mini board code library

// Set the 12-bit combination (4 x 3 bits) for a jackpot win here...
#define WINNING_COMBINATION  0xFFF   // = 111 111 111 111 (binary)

// Define states for the software state machine...
#define WAITING_FOR_BUTTON_PRESS         0
#define SHOWING_RANDOM_CHANGING_PATTERN  1   // also waiting for button release
#define WAITING_FOR_3_SEC_TIMER_EXPIRY   2   // still showing random pattern
#define DISPLAYING_OUTCOME_OF_PLAY       3

#define BUTTON_IS_PRESSED   (TEST_BIT(PIND, 2) == 0)   // PD2 is button 'A' input


// Function prototype declarations...
unsigned int  UpdateRandomCombination(void);
void  DisplayCombination(unsigned  combo);
void  FlashWinningCombination(void);

unsigned int  lastCombination;    // last displayed symbol combination


int main(void)
{
    unsigned long   waitExpiryTime;
    BOOL  isWinner = FALSE;
    BYTE  state = WAITING_FOR_BUTTON_PRESS;   // initial state

    // Initialize LCD panel
    ;
	;

    // Configure required I/O pins, e.g. LED drive pins
    ;
	;
	
	// Enable internal pull-up resistor on button input (PD2)
	;            

    // Initialise Timer TC1 to use milliseconds() library function, etc
    TC1_initialize();
    GLOBAL_INT_ENABLE();

	// Display start-up messages on LCD (prompt user to play game)
    ;
	;
	;
	;

    while ( TRUE )   // loop forever
    {
        if (state == WAITING_FOR_BUTTON_PRESS)
        {
            if (BUTTON_IS_PRESSED)    // button 'A' is pressed
            {
				// prepare display for next state
                ;
				;
                state = SHOWING_RANDOM_CHANGING_PATTERN;  // go to next state
            }
        }
		
        if (state == SHOWING_RANDOM_CHANGING_PATTERN)
        {
            UpdateRandomCombination();  

            if (!BUTTON_IS_PRESSED)   // button 'A' released
            {
                waitExpiryTime = milliseconds() + 3000;   // End time of 3 sec wait (ms)
                state = WAITING_FOR_3_SEC_TIMER_EXPIRY;   // go to next state
            }
        }
		
        if (state == WAITING_FOR_3_SEC_TIMER_EXPIRY)
        {
            lastCombination = UpdateRandomCombination();   // Capture last combination shown

            if (milliseconds() >= waitExpiryTime)   //  3 seconds is up...
            {
                if (lastCombination == WINNING_COMBINATION) 
                {
                    isWinner = TRUE;
					// Display winner message (on LCD bottom line)
					;
					;
					// Flash LED(s) -- (optional)
					;
                }
                else  // any other pattern loses
                {
                    isWinner = FALSE;
                    // Display loser message  (on LCD bottom line)
					;
					;
                }
                waitExpiryTime = milliseconds() + 5000;  // End time of 5 sec wait (ms)
                state = DISPLAYING_OUTCOME_OF_PLAY;      // go to next state
            }
        }
		
        if (state == DISPLAYING_OUTCOME_OF_PLAY)
        {
            if (isWinner)  
			{
				FlashWinningCombination();
				
				// Flash LED(s) -- optional  (call a function -- TBD)
				;
			}

            if (milliseconds() >= waitExpiryTime)   // 5 seconds is up...
            {
                // LED(s) off 
				;
                // Display Prompt for user to play another game
				;
				;
				;
                state = WAITING_FOR_BUTTON_PRESS;   // Start over
            }
        }
    }
}


/*``````````````````````````````````````````````````````````````````````````````````
 * Function to generate a random 12-bit number and display it as a combination of
 * four symbols, where each symbol is represented by 3 bits -- hence 12 bits total.
 *
 * The random combination is updated every 200ms (for an update rate of 5/sec).
 * The function must be called *frequently* from the main loop.
 * If the 200ms update interval has not expired, the function just exits.
 *
 * Return value is the last combination generated (12-bit number in unsigned int).
 */
unsigned int  UpdateRandomCombination()
{
    static unsigned long  periodEndTime;   // static so it is retained
    static unsigned int   combination;     // return value
	
    if (milliseconds() >= periodEndTime)
    {
        combination = rand();   // Get pseudo-random number (16-bit unsigned int)
		
		DisplayCombination(combination);
		
        periodEndTime = milliseconds() + 200;   // start next 200ms period
    }

    return  combination;
}


/*``````````````````````````````````````````````````````````````````````````````````
 * Function to display a given combination on the LCD top line.
 *
 * The argument (combo) is a 12-bit value representing 4 symbols, each 3-bits.
 * Each 3-bit symbol is a code corresponding to a character in the LCD CG ROM table.
 * You can use 8 ordinary (boring) letters and digits, or Japanese or Greek letters.
 * (See character-generator ROM table in LCD module data-sheet.)
 *
 * Note: The array ASCII_code_for_symbol[] is not needed if the 8 characters chosen 
 *       have successive codes in the LCD CG table. The char codes can be calculated.
 *       The array lets you choose any arbitrary characters from the CG ROM table.
 */
void  DisplayCombination(unsigned  combo)
{
	// Below, define ASCII (extended) codes for 8 symbols to be used in the game:
	static char  ASCII_code_for_symbol[] =  
	        { 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88 };
			
	int    i;             // loop counter
	BYTE   symbol;        // 3-bit value representing a symbol in combination
	char   ASCII_code;    // 8-bit character code corresponding to symbol (in CG ROM)
	BYTE   column = 3;    // LCD column to write character (begin at col 3)

	for (i = 0;  i < 4;  i++)   // Loop to display 4 symbols...
	{
		symbol = combo & 0b111;   // Mask off (next) 3 LS bits of combo
		
		// shift next 3 bits of combo into the rightmost 3 bit places
		;    
		
		// look up the array ASCII_code_for_symbol[] to find the character code for symbol
		;  
		
		// Write the character in the correct place on the LCD screen (top line)
		;   // lcd_cursor_posn(0, ???);
		;   // lcd_write_char( ??? );
	}
}


/*``````````````````````````````````````````````````````````````````````````````````
 * Function to flash winning combination on LCD and (optionally) flashing LED(s).
 * The LCD data is flashed at 2Hz, so the half-period is 250ms.
 * The function must be called *frequently* from the main loop.
 * If the 250ms timer has not expired, the function just exits.
 */
void  FlashWinningCombination( void )
{
    static unsigned long  halfPeriodEndTime;
    static BYTE toggle = 0;    // value alternates each half period (0-1-0-1-0-1...)

    if (milliseconds() > halfPeriodEndTime)  // end of half-period 
    {
        // toggle the LCD (top line) between BLANK and WINNING_COMBINATION string
        if (toggle == 0)
        {
            // Display blanks (spaces) in the 4 symbol positions
			;
			;
			;
            toggle = 1;
        }
        else
        {
            // Display winning combination (4 symbols)
			DisplayCombination(WINNING_COMBINATION);
            toggle = 0;
        }

        halfPeriodEndTime = milliseconds() + 250;   // start next 250ms interval
    }
}

/*** end of file ***/
