/**
 * File:  lib_avrSerial.h
 *
 * Definitions and declarations for the AVR "X-mini" Serial Port Function Library.
 *
 */
#ifndef  _LIB_AVR_SERIAL_H    // Avoid multiple inclusion of this file
#define  _LIB_AVR_SERIAL_H

#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "lib_avrXmini.h"     // def's for X-mini peripheral library

// These macros define aliases (nick-names) for various low-level UART functions:
#define putByte(b)     UART_PutByte(b)
#define putChar(c)     UART_PutByte(c)
#define putString(s)   UART_PutString(s)
#define putNewLine()   UART_PutString("\n")

#ifndef ASCII_NUL    // Common ASCII control codes
//````````````````````````````````````````````````````
#define ASCII_NUL       0    // NUL
#define ASCII_ACK       6    // Acknowledge
#define ASCII_BS        8    // Backspace, Ctrl+H
#define ASCII_HT        9    // Horiz. Tab
#define ASCII_TAB       9    // Horiz. Tab (alt.)
#define ASCII_LF       10    // Line Feed
#define ASCII_CR       13    // Carriage Return [Enter]
#define ASCII_DC2      18    // Device Control 2, Ctrl+R
#define ASCII_NAK      21    // Negative acknowledge
#define ASCII_CAN      24    // Cancel line, Ctrl+X
#define ASCII_ESC      27    // Escape, Ctrl+Z ?
#define ASCII_SP       32    // SPACE
#endif
// Alternative names for ASCII codes
#ifndef SPACE
#define NUL            0     // ASCII NUL code (alt.)
#define SPACE          32    // ASCII space code (alt.)
#endif


/**
 * Function:  putDecimal()
 *
 * Description:
 *
 * Entry arg's:
 *
 * Return value:
 *
 * Usage examples:
 *
 */
void  putDecimal(unsigned uval, BYTE nplaces);


/**
 * Function:  getString()
 *
 * Description:
 *
 * Entry arg's:
 *
 * Return value:
 *
 * Usage examples:
 *
 */
BOOL  getString(char *str, BYTE maxlen);


#endif  //  _LIB_AVR_SERIAL_H
